% problem parameters:
C=5;
xmin = [-1*C ; -1*C]; xmax = -xmin; 
dt = .01;
discount = .99;
us = [-1:.2:1];

modes = {'nn', 'delaunay','lqr'}; 
mode_colors = ['bgr'];
ds = [10 20 50]; %100];

figure_number = 100;
for mode_idx = 1:length(modes)
    mode = modes{mode_idx};
    for d_idx = 1:length(ds)
        d = ds(d_idx);
        
        if(strcmp(mode,'lqr'))
            [K, Ps] = lqr_for_brick(dt, discount); % note we pass in the discount factor, which is easily incorporated into the LQR back-ups (yet voids simply calling matlab's dlqr) and which ensures LQR and discretization are solving the same problem
            plot_V(xmin, xmax, d, figure_number, mode, Ps);
        else
            [T, g, delaunayparams] = build_discretized_MDP_for_brick(mode, xmin, xmax, d, us, dt, discount);
            V = value_iteration(g, T, discount);
            plot_V(xmin, xmax, d, figure_number, mode, V);
        end
        


        
        
        % check performance:

        x1s =  -[-.5 0 .5];
        x2s =   [-.5 0 .5];
        H = 1000;
        figure(1000+figure_number); hold on; title(['trajectories under mode = ' mode ' , d = ' num2str(d)]); xlabel('x'); ylabel('\dot{x}')
        i=1;
        for x1=x1s
            j=1;
            for x2=x2s
                xstart = [x1;x2];
                if(strcmp(mode,'lqr'))
                    [V_achieved(i,j) xtraj utraj ts] = execute_lqr(K, xstart, dt, H, discount);
                else
                    [V_achieved(i,j) xtraj utraj ts] = execute_greedy_policy_wrt_V(V, xstart, us, dt, H, discount, mode, xmin, xmax, d, delaunayparams);
                end
                plot(xtraj(1,:),xtraj(2,:)); plot(xtraj(1,1),xtraj(2,1),'xr'); plot(xtraj(1,end), xtraj(2,end), 'xg');
                j=j+1;
            end
            i=i+1;
        end
        
        figure_number = figure_number + 1;


        V_achieved_avg(mode_idx, d_idx) = mean(mean(V_achieved));
    end
end



figure; hold on;
for mode_idx = 1:length(modes)
    plot(ds, V_achieved_avg(mode_idx,:), mode_colors(mode_idx));
end


        
