function [statedot, dfdx, dfdu] = cartpole_dynamics(t, state, u, params)

%state is [x, theta, xdot, thetadot]'
%parameters:
%g - gravity
%mp - pole mass
%mc - cart mass
%lg - pin to pole CG distance
%Ip - pole inertia ABOUT CG!!
%u - force on cart

%theta - 0 when pole hanging straight down; counterclockwise = positive

%default parameters
g = 9.81;
mp = 1;
mc = 10;
lg = 1;
Ip = 0;

if(nargin == 4)
    if(isfield(params, 'g'))
        g = params.g;
    end
    if(isfield(params, 'mp'))
        mp = params.mp;
    end
    if(isfield(params, 'mc'))
        mc = params.mc;
    end
    if(isfield(params, 'lg'))
        lg = params.lg;
    end
    if(isfield(params, 'Ip'))
        Ip = params.Ip;
    end
end

x = state(1);
th = state(2);
xdot = state(3);
thdot = state(4);

c = cos(th);
s = sin(th);

H = [mc + mp    mp*lg*c;
     mp*lg*c    mp*lg^2 + Ip];
 
C = zeros(2);
C(1,2) = -mp*lg*thdot*s;

G = [0; mp*g*lg*s];

B = [1; 0];

acc = inv(H)*(B*u - G - C*[xdot; thdot]);

statedot = [xdot; thdot; acc];

%gradients
if(nargout>=2)
    determ = det(H);
    foo = acc*determ;
    dfdx = zeros(4);
    dfdx(1,3) = 1; %xdot wrt xdot
    dfdx(2,4) = 1; %thdot wrt thdot
    dfdx(3,2) = (determ*(mp^2*lg^3*thdot^2*c+Ip*mp*lg*thdot^2*c+mp^2*g*lg^2*cos(2*th)) - ...
        foo(1)*mp^2*lg^2*sin(2*th))/(determ^2); %xdd wrt th
    dfdx(4,2) = (determ*(mp*lg*s*u-mp^2*lg^2*thdot^2*cos(2*th)-mc*mp*g*lg*c-mp^2*g*lg*c) - ...
        foo(2)*mp*mp*lg*lg*sin(2*th))/(determ^2); %thdd wrt th
    dfdx(3,4) = (mp^2*lg^3*2*thdot*s + 2*Ip*mp*lg*thdot*s)/determ; %xdd wrt thdot
    dfdx(4,4) = (-mp^2*lg^2*thdot*sin(2*th))/determ;
    dfdt = zeros(4,1);
    dfdu = [0; 0; (mp*lg^2 + Ip)/determ; -mp*lg*c/determ];
    df = {[dfdt dfdx dfdu]};
end