%nonlinear optimization of swing up task
%Author: Woody Hoburg
%Modified by: Pieter Abbeel

x0 = zeros(4,1);    %hanging straight down

% initialization of open loop control sequence (utape)
alpha_u = zeros(1,20); %set number of policy parameters here
alpha_u(1) = 1; %give 'er a kick to start

T = 4;  %total duration in seconds

figure(1); axis([-2.5 2.5 0 2*pi]); clf(1);

%gradient descent prep
eta = .1*ones(size(alpha_u));
iter = 0;
Jbest = Inf;
alphaold = alpha_u;
djdaold = zeros(size(alpha_u));
while(iter < 500 && max(eta) > 1e-5)
    [J,dJda,tout,xout] = rtrl(@cartpole_dynamics,@control_utape_zoh,@costfun,@finalcostfun,T,x0,alpha_u);
        
    plot(xout(:,1), xout(:,2)); axis([-2.5 2.5 -1.5*pi 1.5*pi]); drawnow;
    iter = iter+1;
    
    if(J<Jbest)
        eta(sign(dJda)==sign(djdaold)) = eta(sign(dJda)==sign(djdaold))*1.1;
        eta(sign(dJda)~=sign(djdaold)) = eta(sign(dJda)~=sign(djdaold))/2;
        alphaold = alpha_u;
        alpha_u = alpha_u - eta.*dJda;
        Jbest = J;
        djdaold = dJda;
    else
        eta = eta/2;
        alpha_u = alphaold - eta.*djdaold;
    end
    disp(['J = ', num2str(J), '; eta_max = ', num2str(max(eta)), '; iter = ', num2str(iter)]);
end

utape = alphaold;

sim_traj(tout, xout, @cartpole_draw);

figure; 
subplot(5,1,1); plot(xout(1,:)); ylabel('x');
subplot(5,1,2); plot(xout(2,:)); ylabel('theta');
subplot(5,1,3); plot(xout(3,:)); ylabel('xdot');
subplot(5,1,4); plot(xout(4,:)); ylabel('thetadot');
subplot(5,1,5); plot(utape); ylabel('u');
    


